def test(env_vars):
    print(env_vars.domain)


def test_dynamic(env_vars):
    env_vars.put("hello", "2021")
    print(env_vars.get("hello"))


def test_your_name(env_vars_your_name):
    print(env_vars_your_name.your_var)



#!/usr/bin/python
# encoding=utf-8

import jmespath
from loguru import logger

from tep.client import request


def test_post(faker_ch, url, login):
    # description
    logger.info("test post")
    # data
    fake = faker_ch
    # request
    response = request(
        "post",
        url=url("/api/users"),
        headers=login.jwt_headers,
        json={
            "name": fake.name()
        }
    )
    # assert
    assert response.status_code < 400
    # extract
    user_id = jmespath.search("id", response.json())



#!/usr/bin/python
# encoding=utf-8

""" Can only be modified by the administrator. Only fixtures are provided.
"""

from tep.dao import mysql_engine
from tep.fixture import *


@pytest.fixture(scope="session")
def env_vars(config):
    class Clazz(TepVars):
        env = config["env"]

        """Variables define start"""
        # Environment and variables
        mapping = {
            "qa": {
                "domain": "https://qa.com",
                "mysql_engine": mysql_engine("127.0.0.1",  # host
                                             "2306",  # port
                                             "root",  # username
                                             "123456",  # password
                                             "qa"),  # db_name
            },
            "release": {
                "domain": "https://release.com",
                "mysql_engine": mysql_engine("127.0.0.1",
                                             "2306",
                                             "root",
                                             "123456",
                                             "release"),
            }
            # Add your environment and variables
        }
        # Define properties for auto display
        domain = mapping[env]["domain"]
        mysql_engine = mapping[env]["mysql_engine"]
        """Variables define end"""

    return Clazz()


@pytest.fixture
def project_level():
    pass



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Don
@Date    :  2020/12/30 9:30
@Desc    :  Provide some fixtures
"""

import os

import jmespath
import pytest
import yaml
from faker import Faker
from loguru import logger


class Project:
    dir = ""


def _project_dir_assigned():
    while True:
        if Project.dir:
            return True


def pytest_sessionstart(session):
    Project.dir = session.config.cache.get("project_dir", None)


@pytest.fixture(scope="session")
def url(env_vars):
    def domain_and_uri(uri):
        if not uri.startswith("/"):
            uri = "/" + uri
        return env_vars.domain + uri

    return domain_and_uri


@pytest.fixture(scope="session")
def faker_ch():
    return Faker(locale="zh_CN")


@pytest.fixture(scope="session")
def faker_en():
    return Faker()


@pytest.fixture(scope="session")
def pd():
    import pandas
    return pandas


@pytest.fixture(scope="session")
def config():
    if _project_dir_assigned():
        config_path = os.path.join(Project.dir, "conf.yaml")
        with open(config_path, "r", encoding="utf-8") as f:
            conf = yaml.load(f.read(), Loader=yaml.FullLoader)
            return conf


@pytest.fixture(scope="session")
def files_dir():
    if _project_dir_assigned():
        return os.path.join(Project.dir, "files")


class TepVars:
    def __init__(self):
        self.vars_ = {}

    def put(self, key, value):
        self.vars_[key] = value

    def get(self, key):
        value = ""
        try:
            value = self.vars_[key]
        except KeyError:
            logger.error(f"env_vars doesnt have this key: {key}")
        return value


_jmespath_import_placeholder = jmespath.search("abc", "abc")



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Don
@Date    :  8/14/2020 9:16 AM
@Desc    :  
"""

import os
import shutil
import tempfile

import allure_commons
from allure_commons.logger import AllureFileLogger
from allure_pytest.listener import AllureListener
from allure_pytest.plugin import cleanup_factory

from tep.fixture import Project
from tep.funcs import current_time

allure_temp = tempfile.mkdtemp()


class Plugin:
    @staticmethod
    def pytest_addoption(parser):
        parser.addoption(
            "--tep-reports",
            action="store_const",
            const=True,
            help="Create tep allure HTML reports."
        )

    @staticmethod
    def _tep_reports(config):
        if config.getoption("--tep-reports") and not config.getoption("allure_report_dir"):
            return True
        else:
            return False

    @staticmethod
    def pytest_configure(config):
        if Plugin._tep_reports(config):
            test_listener = AllureListener(config)
            config.pluginmanager.register(test_listener)
            allure_commons.plugin_manager.register(test_listener)
            config.add_cleanup(cleanup_factory(test_listener))

            clean = config.option.clean_alluredir
            file_logger = AllureFileLogger(allure_temp, clean)
            allure_commons.plugin_manager.register(file_logger)
            config.add_cleanup(cleanup_factory(file_logger))

    @staticmethod
    def pytest_sessionfinish(session):
        if Plugin._tep_reports(session.config):
            reports_dir = os.path.join(Project.dir, "reports")
            new_report = os.path.join(reports_dir, "report-" + current_time().replace(":", "-").replace(" ", "-"))
            if os.path.exists(reports_dir):
                his_reports = os.listdir(reports_dir)
                if his_reports:
                    latest_report_history = os.path.join(reports_dir, his_reports[-1], "history")
                    shutil.copytree(latest_report_history, os.path.join(allure_temp, "history"))
            os.system(f"allure generate {allure_temp} -o {new_report}  --clean")
            shutil.rmtree(allure_temp)



#!/usr/bin/python
# encoding=utf-8

""" Can only be modified by the administrator. Only fixtures are provided.
"""

import os

import pytest

# Initial
_project_dir = os.path.dirname(os.path.abspath(__file__))


@pytest.fixture(scope="session", autouse=True)
def _project_cache(request):
    request.config.cache.set("project_dir", _project_dir)


# Auto import fixtures
_fixtures_dir = os.path.join(_project_dir, "fixtures")
for root, _, files in os.walk(_fixtures_dir):
    for file in files:
        if os.path.isfile(os.path.join(root, file)):
            if file.startswith("fixture_") and file.endswith(".py"):
                _fixture_name, _ = os.path.splitext(file)
                exec(f"from fixtures.{_fixture_name} import *")



from tep.client import request
from tep.fixture import *


def _jwt_headers(token):
    return {"Content-Type": "application/json", "authorization": f"Bearer {token}"}


@pytest.fixture(scope="session")
def login(url):
    # Code your login
    logger.info("Administrator login")
    response = request(
        "post",
        url=url("/api/users/login"),
        headers={"Content-Type": "application/json"},
        json={
            "username": "admin",
            "password": "123456",
        }
    )
    assert response.status_code < 400
    response_token = jmespath.search("token", response.json())

    class Clazz:
        token = response_token
        jwt_headers = _jwt_headers(response_token)

    return Clazz



#!/usr/bin/python
# encoding=utf-8

""" Please define your own fixture.
"""

from tep.fixture import *


@pytest.fixture(scope="session")
def env_vars_your_name(config):
    class Clazz:
        env = config["env"]

        # Environment and variables
        mapping = {
            "qa": {
                "your_var": "123",
            },
            "release": {
                "your_var": "456",
            }
            # Add your environment and variables
        }
        # Define properties for auto display
        your_var = mapping[env]["your_var"]

    return Clazz()


@pytest.fixture
def share_your_name():
    pass



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Don
@Date    :  7/17/2020 3:49 PM
@Desc    :  Command line.
"""

import argparse
import sys

from tep import __description__, __version__
from tep.scaffold import init_parser_scaffold, main_scaffold, update_scaffold


def main():
    """Parse command line options and run commands.
    """
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument(
        "-V", "--version", dest="version", action="store_true", help="show version"
    )
    subparsers = parser.add_subparsers(help="sub-command help")
    sub_parser_scaffold = init_parser_scaffold(subparsers)

    if len(sys.argv) == 1:
        # tep
        parser.print_help()
        sys.exit(0)
    elif len(sys.argv) == 2:
        # print help for sub-commands
        if sys.argv[1] in ["-V", "--version"]:
            # tep -V
            print(f"{__version__}")
        elif sys.argv[1] in ["-h", "--help"]:
            # tep -h
            parser.print_help()
        elif sys.argv[1] == "startproject":
            # tep startproject
            sub_parser_scaffold.print_help()
        elif sys.argv[1] in ["-U", "--upgrade"]:
            # tep -U
            update_scaffold()
        sys.exit(0)

    args = parser.parse_args()

    if args.version:
        print(f"{__version__}")
        sys.exit(0)

    if sys.argv[1] == "startproject":
        main_scaffold(args)



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Don
@Date    :  7/25/2020 2:02 PM
@Desc    :
"""

import decimal
import json
import time

import requests
import urllib3
from loguru import logger
from requests import sessions

from tep.funcs import NpEncoder

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def request_encapsulate(req):
    def send(*args, **kwargs):
        start = time.process_time()
        response = req(*args, **kwargs)
        end = time.process_time()
        elapsed = str(decimal.Decimal("%.3f" % float(end - start))) + "s"

        # log
        try:
            log4a = {"method": args[0]}
            for k, v in kwargs.items():
                # if not json, str()
                try:
                    json.dumps(v)
                except TypeError:
                    v = str(v)
                log4a.setdefault(k, v)
            log4a.setdefault("status", response.status_code)
            log4a.setdefault("response", response.text)
            log4a.setdefault("elapsed", elapsed)
            logger.info(json.dumps(log4a, ensure_ascii=False, cls=NpEncoder))
        except AttributeError:
            logger.error("request failed")
        except TypeError:
            logger.warning(log4a)

        return response

    return send


@request_encapsulate
def request(method, url, **kwargs):
    """Constructs and sends a :class:`Request <Request>`.

    :param method: method for the new :class:`Request` object: ``GET``, ``OPTIONS``, ``HEAD``, ``POST``, ``PUT``, ``PATCH``, or ``DELETE``.
    :param url: URL for the new :class:`Request` object.
    :param params: (optional) Dictionary, list of tuples or bytes to send
        in the query string for the :class:`Request`.
    :param data: (optional) Dictionary, list of tuples, bytes, or file-like
        object to send in the body of the :class:`Request`.
    :param json: (optional) A JSON serializable Python object to send in the body of the :class:`Request`.
    :param headers: (optional) Dictionary of HTTP Headers to send with the :class:`Request`.
    :param cookies: (optional) Dict or CookieJar object to send with the :class:`Request`.
    :param files: (optional) Dictionary of ``'name': file-like-objects`` (or ``{'name': file-tuple}``) for multipart encoding upload.
        ``file-tuple`` can be a 2-tuple ``('filename', fileobj)``, 3-tuple ``('filename', fileobj, 'content_type')``
        or a 4-tuple ``('filename', fileobj, 'content_type', custom_headers)``, where ``'content-type'`` is a string
        defining the content type of the given file and ``custom_headers`` a dict-like object containing additional headers
        to add for the file.
    :param auth: (optional) Auth tuple to enable Basic/Digest/Custom HTTP Auth.
    :param timeout: (optional) How many seconds to wait for the server to send data
        before giving up, as a float, or a :ref:`(connect timeout, read
        timeout) <timeouts>` tuple.
    :type timeout: float or tuple
    :param allow_redirects: (optional) Boolean. Enable/disable GET/OPTIONS/POST/PUT/PATCH/DELETE/HEAD redirection. Defaults to ``True``.
    :type allow_redirects: bool
    :param proxies: (optional) Dictionary mapping protocol to the URL of the proxy.
    :param verify: (optional) Either a boolean, in which case it controls whether we verify
            the server's TLS certificate, or a string, in which case it must be a path
            to a CA bundle to use. Defaults to ``True``.
    :param stream: (optional) if ``False``, the response content will be immediately downloaded.
    :param cert: (optional) if String, path to ssl client cert file (.pem). If Tuple, ('cert', 'key') pair.
    :return: :class:`Response <Response>` object
    :rtype: requests.Response

    Usage::

      >>> import requests
      >>> req = requests.request('GET', 'https://httpbin.org/get')
      >>> req
      <Response [200]>
    """

    # By using the 'with' statement we are sure the session is closed, thus we
    # avoid leaving sockets open which can trigger a ResourceWarning in some
    # cases, and look like a memory leak in others.
    with sessions.Session() as session:
        return session.request(method=method, url=url, **kwargs)



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Don
@Date    :  7/23/2020 8:12 PM
@Desc    :
"""

import os
import shutil
import sys

from loguru import logger

from tep import __version__


def init_parser_scaffold(subparsers):
    sub_parser_scaffold = subparsers.add_parser(
        "startproject", help="Create a new project with template structure."
    )
    sub_parser_scaffold.add_argument(
        "project_name", type=str, nargs="?", help="Specify new project name."
    )
    return sub_parser_scaffold


def create_scaffold(project_name):
    """ Create scaffold with specified project name.
    """
    if os.path.isdir(project_name):
        logger.warning(
            f"Project folder {project_name} exists, please specify a new project name."
        )
        return 1
    elif os.path.isfile(project_name):
        logger.warning(
            f"Project name {project_name} conflicts with existed file, please specify a new one."
        )
        return 1

    logger.info(f"Create new project: {project_name}")
    print(f"Project root dir: {os.path.join(os.getcwd(), project_name)}\n")

    def create_folder(path):
        os.makedirs(path)
        msg = f"Created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"Created file: {path}"
        print(msg)

    def copy_demo_file(relative_path, filename):
        tep_demo_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "demo")
        src = os.path.join(tep_demo_path, relative_path, filename)
        des = os.path.join(project_name, relative_path)
        shutil.copy(src, des)
        msg = f"Created file: {os.path.join(des, filename)}"
        print(msg)

    create_folder(project_name)
    create_folder(os.path.join(project_name, "fixtures"))
    create_folder(os.path.join(project_name, "tests"))
    create_folder(os.path.join(project_name, "tests", "sample"))
    create_folder(os.path.join(project_name, "tests", "sample", "case_reuse"))
    create_folder(os.path.join(project_name, "files"))

    copy_demo_file("", ".gitignore")
    copy_demo_file("", "conf.yaml")
    copy_demo_file("", "conftest.py")
    copy_demo_file("", "pytest.ini")
    copy_demo_file("", "requirements.txt")

    create_file(os.path.join(project_name, "fixtures", "__init__.py"))
    copy_demo_file("fixtures", "fixture_admin.py")
    copy_demo_file("fixtures", "fixture_login.py")
    copy_demo_file("fixtures", "fixture_your_name.py")

    create_file("tests", "__init__.py")
    copy_demo_file(os.path.join("tests", "sample"), "login_test.py")
    copy_demo_file(os.path.join("tests", "sample"), "mysql_test.py")
    copy_demo_file(os.path.join("tests", "sample"), "post_test.py")
    copy_demo_file(os.path.join("tests", "sample"), "var_reuse_test.py")

    create_file(os.path.join(project_name, "tests", "sample", "case_reuse", "__init__.py"))
    copy_demo_file(os.path.join("tests", "sample", "case_reuse"), "a_test.py")
    copy_demo_file(os.path.join("tests", "sample", "case_reuse"), "reuse_a_test.py")


def update_scaffold():
    def create_folder(path):
        os.makedirs(path)
        msg = f"Created folder: {path}"
        print(msg)

    def copy_demo_file(relative_path, filename, des):
        tep_demo_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "demo")
        src = os.path.join(tep_demo_path, relative_path, filename)
        shutil.copy(src, des)
        msg = f"Updated file: {os.path.join(des, filename)}"
        print(msg)

    if not os.path.exists("conftest.py"):
        logger.warning("tep project not found, please relocate to the tep project which contains a conftest.py file.")
        return

    if __version__ >= "0.6.0":
        version_dir = f"version{__version__}"
        version_fixtures_dir = os.path.join(version_dir, "fixtures-upgrade")
        if os.path.exists(version_dir):
            logger.warning("Version directory exists, please delete first.")
            return
        create_folder(version_dir)

        create_folder(version_fixtures_dir)
        copy_demo_file("fixtures", "fixture_admin.py", des=version_fixtures_dir)
        copy_demo_file("fixtures", "fixture_login.py", des=version_fixtures_dir)
        copy_demo_file("fixtures", "fixture_your_name.py", des=version_fixtures_dir)

        copy_demo_file("", "conftest.py", des=os.getcwd())
    else:
        logger.info("There is no need to upgrade scaffold, please check the version of tep.")
        return


def main_scaffold(args):
    sys.exit(create_scaffold(args.project_name))



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Don
@Date    :  9/2/2020 11:32 AM
@Desc    :  
"""

from loguru import logger
from sqlalchemy import create_engine
from texttable import Texttable


def mysql_engine(host, port, user, password, db):
    return create_engine(f"mysql+pymysql://{user}:{password}@{host}:{port}/{db}")


def print_db_table(data_frame):
    tb = Texttable()
    tb.header(data_frame.columns.array)
    tb.set_max_width(0)
    # text * cols
    tb.set_cols_dtype(['t'] * data_frame.shape[1])
    tb.add_rows(data_frame.to_numpy(), header=False)
    logger.info(tb.draw())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tep',
 'tep.demo',
 'tep.demo.fixtures',
 'tep.demo.tests',
 'tep.demo.tests.sample',
 'tep.demo.tests.sample.case_reuse']

package_data = \
{'': ['*']}

install_requires = \
['allure-pytest>=2.8.16,<3.0.0',
 'allure-python-commons>=2.8.16,<3.0.0',
 'faker>=4.1.1,<5.0.0',
 'jmespath>=0.10.0,<0.11.0',
 'loguru>=0.5.1,<0.6.0',
 'pyyaml>=5.3.1,<6.0.0',
 'requests>=2.24.0,<3.0.0',
 'urllib3>=1.25.9,<2.0.0']

entry_points = \
{'console_scripts': ['tep = tep.cli:main'],
 'pytest11': ['tep = tep.plugin:Plugin']}

setup_kwargs = {
    'name': 'tep',
    'version': '0.6.1',
    'description': 'tep is a testing tool to help you write pytest more easily. Try Easy Pytest!',
    'long_description': '# tep\n\ntep is a testing tool to help you write pytest more easily. Try Easy Pytest!\n\n# Design Philosophy\n\n- Simple is better\n- Everyone can write automation in python\n\n# Installation\n\n`tep` is developed with Python, it supports Python `3.6+` and most operating systems.\n\n`tep` is available on [`PyPI`](https://pypi.python.org/pypi) and can be installed through `pip`.\n\n```\n$ pip install tep\n```\n\nor domestic mirror.\n\n```\n$ pip --default-timeout=600 install -i https://pypi.tuna.tsinghua.edu.cn/simple tep\n```\n\n# Check Installation\n\nWhen tep is installed, tep command will be added in your system.\n\nTo see `tep` version:\n\n```\n$ tep -V  # tep --version\n0.2.3\n```\n\n# Usage\n\nIf you want to know more usages, you can read [pytest docs](https://docs.pytest.org/).\n\nYou know pytest.\n\nYou know tep.\n',
    'author': 'dongfanger',
    'author_email': 'dongfanger@outlook.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dongfanger/tep',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
